#!/usr/bin/env python
#
# Giacomo.Lozito@gmail.com
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>.

from .UptimeServices import *

def getService(service_name, api_token, filter_opts):
  service = [service for service in Base.BaseUptimeService.__subclasses__() if service_name == service.SERVICE_NAME]
  if service:
    return service[0](api_token, filter_opts)
  else:
    raise Exception(f'Uptime Service {service_name} not supported')

def getSupportedServices():
  return [service.SERVICE_NAME for service in Base.BaseUptimeService.__subclasses__()]

